package com.ai.service;

import dev.langchain4j.community.model.dashscope.QwenEmbeddingModel;
import dev.langchain4j.data.document.Document;
import dev.langchain4j.data.document.loader.FileSystemDocumentLoader;
import dev.langchain4j.data.document.splitter.DocumentByLineSplitter;
import dev.langchain4j.data.embedding.Embedding;
import dev.langchain4j.data.segment.TextSegment;
import dev.langchain4j.model.output.Response;
import dev.langchain4j.store.embedding.EmbeddingMatch;
import dev.langchain4j.store.embedding.EmbeddingSearchRequest;
import dev.langchain4j.store.embedding.EmbeddingSearchResult;
import dev.langchain4j.store.embedding.inmemory.InMemoryEmbeddingStore;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class EmbeddingService {

    @Value("${langchain4j.api-key}")
    private String apiKey;

    @Value("${rag.path}")
    private String ragPath;

    public void embedding(String input) {
        QwenEmbeddingModel embeddingModel = QwenEmbeddingModel.builder().apiKey(apiKey).build();
        InMemoryEmbeddingStore<TextSegment> embeddingStore = new InMemoryEmbeddingStore<>();
        List<Document> documents = FileSystemDocumentLoader.loadDocuments(ragPath);
        for (Document document : documents) {
            DocumentByLineSplitter splitter = new DocumentByLineSplitter(200,30);
            List<TextSegment> segments = splitter.split(document);
            List<Embedding> embeddings = embeddingModel.embedAll(segments).content();
            embeddingStore.addAll(embeddings, segments);
            Response<Embedding> embed = embeddingModel.embed(input);
            EmbeddingSearchRequest builder = EmbeddingSearchRequest.builder().queryEmbedding(embed.content()).build();
            EmbeddingSearchResult<TextSegment> results = embeddingStore.search(builder);
            for (EmbeddingMatch<TextSegment> match : results.matches()) {
                System.out.println(match.embedded().text() + ", 分数为" + match.score());
            }
        }
    }
}
